div.simple_bars div.row {
    position: relative;
    padding: 3px 1%;
    width: 100%;
    overflow: auto;
}

div.simple_bars div.row span.label {
    display: block;
    float: left;
    width: 30%;
    overflow: hidden;
}

div.simple_bars div.row span.barArea {
    display: block;
    float: left;
    width: 60%;
    overflow: hidden;
}

div.simple_bars div.row span.barArea span.bar {
    display: block;
}

div.simple_bars span.bar {
    background-color: #4d89f9;
}

div.simple_bars div.row span.value {
    display: block;
    float: left;
    width: 8%;
    text-align: right;
}
